@@ -27,6 +27,10 @@
 #define IS_DIGIT(c) ((c >= '0' && c <= '9') || (c >= 'A' && c <= 'J'))
 #define EPSILON 0.0000001
 
+#ifdef UNDERCONSTRUCTION
+#define DEBUG
+#endif
+
 /* ignores decimal point and multiply signs, etc, etc */
 static void simpleParser(AbacusWidget w, char * buffer,
 		char* aString, char* bString, char *op)
@@ -552,11 +556,46 @@ void testShift(char * string)
 	for (i = -4; i < 5; i++) {
 		(void) strcpy(yours, my);
 		shiftStringBuffer(yours, i);
-		(void) printf("my %s, i %d, yours %s\n",my, i, yours);
+		(void) printf("my %s, i %d, yours %s\n", my, i, yours);
 	}
 }
 #endif
 
+static void
+stringGroup(int *intGroup, char *group, char v)
+{
+	char temp[512];
+
+	temp[0] = '\0';
+	for (; *intGroup != -2; intGroup++) {
+		if (*intGroup == -1) {
+			(void) strcat(group, " . ");
+		} else {
+			(void) strcpy(temp, group);
+			if (v == 'v')
+				(void) sprintf(group, "%s %02d", temp, *intGroup);
+			else
+				(void) sprintf(group, "%s %03d", temp, *intGroup);
+		}
+	}
+}
+
+#ifdef UNDERCONSTRUCTION
+static int
+placeGroup(int *intGroup)
+{
+	int i = 0;
+
+	for (; *intGroup != -2; intGroup++) {
+		if (*intGroup == -1) {
+			return i;
+		}
+		i++;
+	}
+	return i;
+}
+#endif
+
 /* Set string given change in rail (decimal position) */
 static void setStringBuffer(AbacusWidget w, char * string, int aux,
 		int place, int lower, int upper)
@@ -904,6 +943,7 @@ static void rootGroup(AbacusWidget w, char* string, int root, int** group)
 }
 
 #ifdef DEBUG
+#ifdef KOJIMA
 /* Implement an iteration of Newton's Method or Halley's Method */
 static double rootAdvance(double x, double p, int n)
 {
@@ -981,6 +1021,7 @@ static void testRoot(AbacusWidget w, double guess, int position, int root)
 	(void) printf("incr root = %g\n", c);
 #endif
 }
+#endif
 
 /* Figure out next intGroup value */
 static int intGroupValue(AbacusWidget w, int position) {
@@ -1001,39 +1042,51 @@ static int intGroupValue(AbacusWidget w, int position) {
 	return w->abacus.intGroup[newPosition];
 }
 
+typedef struct {
+	int digitAdd;
+	int orderInc; /* cbrt only */
+	int groupInc;
+} digitStruct;
+
+#if 0
+typedef struct {
+	int groupInc;
+	int groupSum;
+} groupStruct;
+#endif
+
 /* This will test Lee's method for calculating sqrt via Newton's Method */
 static int testSqrtAdvance(AbacusWidget w, int power,
 		int *digitSum, int *groupInc, int *groupSum)
 {
 	int digitAdd = 1;
 	int i = 0;
-	int digitSumOrig = *digitSum;
-	int groupSumOrig = *groupSum;
+	digitStruct *ds;
 
-	*groupInc += 1;
+	ds = (digitStruct *) calloc(w->abacus.base - 1, sizeof (digitStruct));
 #ifdef DEBUG
 	(void) printf("remainder power = %d\n", power);
 #endif
-	do {
-		i++;
-#ifdef DEBUG
-		(void) printf("%d: %d %d\n", i, digitAdd, *groupInc);
+	while (*groupSum + *groupInc <= power) {
+		*groupInc += digitAdd;
+#if UNDERCONSTRUCTION
+		ds[i].digitAdd = digitAdd;
+		ds[i].groupInc = *groupInc;
 #endif
+#ifdef DEBUG
+		(void) printf("%d: A-0 field %d, P-O field %d\n",
+			(i + 1), digitAdd, *groupInc);
+#endif
+		i++;
 		*digitSum += digitAdd;
 		*groupSum += *groupInc;
 		digitAdd = 2;
-		*groupInc += digitAdd;
-	} while (*groupSum + *groupInc <= power);
-	*groupInc -= 2;
-	if (i == 1) {
-		(*groupInc)--;
-		*digitSum = digitSumOrig;
-		*groupSum = groupSumOrig;
 	}
 #ifdef DEBUG
 	(void) printf("sum %d: %d %d : %d\n",
 		i, *digitSum, *groupInc, *groupSum);
 #endif
+	free(ds);
 	return i;
 }
 
@@ -1053,14 +1106,14 @@ static void testSqrt(AbacusWidget w, int position)
 		digitSum = 0;
 		groupSum = 0;
 #ifdef DEBUG
-		(void) printf("%d: incr power = %d at position %d, intGroup %d\n",
-			i, power, position, groupVal);
+		(void) printf("A-0 digit %d: incr power = %d at position %d, intGroup %d\n",
+			(i + 1), power, position, groupVal);
 #endif
 		digitSum *= w->abacus.base;
 		groupSum *= baseSq;
 		j = testSqrtAdvance(w, power,
 			&digitSum, &groupInc, &groupSum);
-		if (j == 1) {
+		if (j == 0) {
 			digitSum = 0;
 			groupInc = groupInc * w->abacus.base;
 		} else {
@@ -1068,8 +1121,8 @@ static void testSqrt(AbacusWidget w, int position)
 			groupInc = (groupInc + 1) * w->abacus.base;
 		}
 #ifdef DEBUG
-		(void) printf("%d: digitSum = %d, groupSum = %d, power = %d, j = %d\n",
-			i, digitSum, groupSum, power, j);
+		(void) printf("A-O digit %d: digitSum = %d, groupSum = %d, power = %d, j = %d\n",
+			(i + 1), digitSum, groupSum, power, j);
 #endif
 		allDigitSum = allDigitSum * w->abacus.base + digitSum;
 	}
@@ -1093,48 +1146,37 @@ static int testCbrtAdvance(AbacusWidget w, int power, int *digitSum,
 {
 	int digitAdd = 1;
 	int i = 0;
-	int digitSumOrig = *digitSum;
-	int orderSumOrig = *orderSum;
-	int groupSumOrig = *groupSum;
 	int doubleOrder = 0;
 
-	*orderInc += 1;
-	*groupInc += 1;
 #ifdef DEBUG
 	(void) printf("remainder power = %d\n", power);
 #endif
-	do {
-		i++;
+	while (*groupSum + *groupInc <= power) {
+		*orderInc += digitAdd;
+		if (i == 0)
+			doubleOrder = digitAdd;
+		else
+			doubleOrder += *orderInc;
+		*groupInc += doubleOrder;
 #ifdef DEBUG
-		(void) printf("%d: %d %d %d\n",
-			i, digitAdd, *orderInc, *groupInc);
+		(void) printf("%d: left A-0 field %d, right A-O field %d, P-O field %d\n",
+			(i + 1), digitAdd, *orderInc, *groupInc);
 #endif
+		i++;
 		*digitSum += digitAdd;
 		*orderSum += *orderInc;
 		*groupSum += *groupInc;
 		digitAdd = 1;
 		*orderInc += digitAdd;
 		doubleOrder = *orderInc;
-		i++;
 #ifdef DEBUG
-		(void) printf("%d: %d %d\n",
-			i, digitAdd, *orderInc);
+		(void) printf("%d: left A-0 field %d, right A-O field %d\n",
+			(i + 1), digitAdd, *orderInc);
 #endif
+		i++;
 		*digitSum += digitAdd;
 		*orderSum += *orderInc;
 		digitAdd = 2;
-		*orderInc += digitAdd;
-		doubleOrder += *orderInc;
-		*groupInc += doubleOrder;
-	} while (*groupSum + *groupInc <= power);
-	*groupInc -= doubleOrder;
-	*orderInc -= 2;
-	if (i == 2) {
-		*orderInc -= 2;
-		(*groupInc)--;
-		*digitSum = digitSumOrig;
-		*orderSum = orderSumOrig;
-		*groupSum = groupSumOrig;
 	}
 #ifdef DEBUG
 	(void) printf("sum %d: %d : %d %d : %d %d\n", i, *digitSum,
@@ -1161,15 +1203,15 @@ static void testCbrt(AbacusWidget w, int position)
 		digitSum = 0;
 		groupSum = 0;
 #ifdef DEBUG
-		(void) printf("%d: incr power = %d at position %d, intGroup %d\n",
-			i, power, position, groupVal);
+		(void) printf("left A-0 digit %d: incr power = %d at position %d, intGroup %d\n",
+			(i + 1), power, position, groupVal);
 #endif
 		digitSum *= w->abacus.base;
 		orderSum *= w->abacus.base;
 		groupSum *= baseCb;
 		j = testCbrtAdvance(w, power, &digitSum,
 			&orderInc, &orderSum, &groupInc, &groupSum);
-		if (j == 2) {
+		if (j == 0) {
 			digitSum = 0;
 			orderInc = orderInc * w->abacus.base;
 		} else {
@@ -1179,8 +1221,8 @@ static void testCbrt(AbacusWidget w, int position)
 		magnitude *= w->abacus.base;
 		groupInc = orderSum * magnitude + orderInc;
 #ifdef DEBUG
-		(void) printf("%d: digitSum = %d, orderSum = %d, groupSum = %d, power = %d, j = %d\n",
-			i, digitSum, orderSum, groupSum, power, j);
+		(void) printf("left A-0 digit %d: digitSum = %d, orderSum = %d, groupSum = %d, power = %d, j = %d\n",
+			(i + 1), digitSum, orderSum, groupSum, power, j);
 #endif
 		allDigitSum = allDigitSum * w->abacus.base + digitSum;
 	}
@@ -1511,7 +1553,9 @@ static Boolean nextPositionRoot(AbacusWidget w, int root,
 #endif
 		w->abacus.qPosition = aPlace;
 #ifdef DEBUG
+#ifdef KOJIMA
 		testRoot(w, w->abacus.qDigit, aPlace, root);
+#endif
 		if (root == 2)
 			testSqrt(w, aPlace);
 		else if (root == 3)
@@ -1537,6 +1581,54 @@ static Boolean nextPositionRoot(AbacusWidget w, int root,
 	return True;
 }
 
+#ifdef UNDERCONSTRUCTION
+/* A little tricky as this is reentrant */
+void teachStepRoot(AbacusWidget w, char * buffer)
+{
+	char buffer1[120] = "";
+	char buffer2[120] = "";
+	char buffer3[120] = "";
+	int base = w->abacus.base;
+	int bottomNumber = w->abacus.decks[BOTTOM].number;
+	double a, b;
+	int root = (w->abacus.op == 'v') ? 2 : 3;
+	int place;
+
+	if (w->abacus.step == 2) {
+		place = placeGroup(w->abacus.intGroup) - 1;
+		(void) sprintf(buffer2,
+			"Add %d to left A-O field, offset digit by corresponding group %d",
+			1, place);
+		drawLineText(w, buffer2, 1);
+		highlightRail(w, 1, place, True);
+		w->abacus.step++;
+	} else if (w->abacus.step == 3) {
+		/* a carry should never be needed */
+		place = placeGroup(w->abacus.intGroup) - 1;
+		setAbacusMove(w, ACTION_MOVE, 1, 0, place, 1);
+		highlightRail(w, 1, place, False);
+		w->abacus.step++;
+	} else if (w->abacus.step == 4) {
+		place = placeGroup(w->abacus.intGroup) - 1;
+		(void) sprintf(buffer2,
+			"Subtract %d from primary field, offset digit by corresponding group %d",
+			1, place);
+		drawLineText(w, buffer2, 1);
+		highlightRail(w, 0, place * root, True);
+		w->abacus.step++;
+	} else if (w->abacus.step == 5) {
+		printf("do P\n");
+		place = (placeGroup(w->abacus.intGroup) - 1) * root;
+		setAbacusMove(w, ACTION_MOVE, 0, 0, place, -1);/*lower 6->5 */
+		highlightRail(w, 0, place, False);
+		w->abacus.step++;
+	} else if (w->abacus.step >= 6) {
+		printf("loop\n");
+		w->abacus.step++;
+	}
+}
+#endif
+
 /* A little tricky as this is reentrant */
 void teachStep(AbacusWidget w, char * buffer, int aux)
 {
@@ -1719,18 +1811,22 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 			w->abacus.reg = 0;
 			(void) strcpy (w->abacus.cString, w->abacus.aString); /* remainder */
 		} else if (w->abacus.op == 'v' || w->abacus.op == 'u') {
+			char group[512];
+
+			group[0] = '\0';
 			rootGroup(w, w->abacus.aString,
 				(w->abacus.op == 'v') ? 2 : 3,
 				&(w->abacus.intGroup));
+			stringGroup(w->abacus.intGroup, group, w->abacus.op);
 			convertStringToAbacus(w, w->abacus.aString, aux);
 			if (w->abacus.leftAux != NULL)
 				convertStringToAbacus(w, (char *) "0.0", 1);
 			if (w->abacus.rightAux != NULL)
 				convertStringToAbacus(w, (char *) "0.0", 2);
 			a = convertToDecimal(base, w->abacus.aString);
-			(void) sprintf(buffer1, "%s root of %s",
+			(void) sprintf(buffer1, "%s root of %s, grouping digits yields (%s)",
 				((w->abacus.op == 'v') ? "Square" : "Cube"),
-				w->abacus.aString);
+				w->abacus.aString, group);
 			if (!w->abacus.lee) {
 				(void) strcat(buffer1,
 					" ... works best with lee option");
@@ -1753,6 +1849,12 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 #endif
 		w->abacus.carry[0] = 0;
 		w->abacus.carry[1] = 0;
+#ifdef UNDERCONSTRUCTION
+	} else if (w->abacus.op == 'v' || w->abacus.op == 'u') {
+		/* Keep separate as its too different */
+		teachStepRoot(w, buffer);
+		return;
+#endif
 	} else if (w->abacus.reg == 0 || (w->abacus.reg < 0 &&
 			((w->abacus.carryStep != 0 && w->abacus.carryStep % 2 == 0 &&
 			w->abacus.carryStep >= 2) ||
@@ -1882,7 +1984,8 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				(w->abacus.op == 'v') ? 2 : 3,
 				&rPosition, &aDigit, &bDigit, &bValue);
 			(void) sprintf(buffer1, "Taking %s Root of %s",
-				(w->abacus.op == 'v') ? "Square" : "Cube", w->abacus.cString);
+				(w->abacus.op == 'v') ? "Square" : "Cube",
+				w->abacus.cString);
 			if (w->abacus.carryStep == 0) {
 				(void) strcat(buffer1, " ... ");
 				(void) strAddInt(buffer1, w->abacus.qDigit);
@@ -1903,7 +2006,7 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 						wap = (AbacusWidget *) w->abacus.leftAux;
 					wa = *wap;
 					bn = wa->abacus.decks[BOTTOM].number;
-					(void) strcat(buffer1, " , register  ");
+					(void) strcat(buffer1, " , register ");
 					(void) strAddInt(buffer1, w->abacus.qDigit);
 					(void) strcat(buffer1, " on first auxiliary, rail ");
 					(void) strAddInt(buffer1, w->abacus.qPosition);
@@ -1921,6 +2024,25 @@ void teachStep(AbacusWidget w, char * buffer, int aux)
 				(void) strcat(buffer1, " ... borrowing 1");
 			}
 			drawLineText(w, buffer1, 0);
+#ifdef LEE_ROOT
+		} else if (w->abacus.op == 'v' || w->abacus.op == 'u') {
+			char group[512];
+
+			group[0] = '\0';
+			done = nextPositionRoot(w,
+				(w->abacus.op == 'v') ? 2 : 3,
+				&rPosition, &aDigit, &bDigit, &bValue);
+			stringGroup(w->abacus.intGroup, group, w->abacus.op);
+			(void) sprintf(buffer1, "Taking %s Root of (%s)",
+				(w->abacus.op == 'v') ? "Square" : "Cube", group);
+				/*w->abacus.cString */
+			(void) strcat(buffer1, " , register ");
+			(void) strAddInt(buffer1, w->abacus.intGroup[0]);
+			(void) strcat(buffer1, " on primary, rail ");
+			drawLineText(w, buffer1, 0);
+			(void) pendingUpdate(w, buffer2, 1,
+				1, w->abacus.intGroup[0], w->abacus.decks[BOTTOM].number);
+#endif
 		}
 		if (!pendingUpdate(w, buffer2, 1, 0, rPosition, bottomNumber))
 			done = False;
